from OpenGL.GL import *
from OpenGL.GLU import *

import vector
import pygame

_screen = None

class Screen(object):
    def __init__(self, (w, h),(n_w, n_h)):
        assert w > 0 and h > 0
        self.current_texture = 0
        self.width = w
        self.height = h
        self.aspect = float(h)/float(w)
        self.ratio = 2.0 / min([w,h])
        self.z = 0.0
        self.dimensions = vector.Vector((n_w, n_h))

    def fill(self, color):
        assert len(color) == 4
        glClearColor(*color)
        glClear(GL_COLOR_BUFFER_BIT)
        glClear(GL_DEPTH_BUFFER_BIT)

    def blit(self, surface, dest):
        glPushMatrix()
        try:
            glTranslatef(dest[0], dest[1], self.z)
        except IndexError:
            raise IndexError, 'bad destination: %s' % dest
        try:
            glRotatef(surface.rotation, 0, 0, 1)
        except AttributeError:
            pass
            
        try:
            scale = surface.scale
            glScalef(scale,scale,scale)
        except AttributeError:
            pass
        glColor4f(*surface.color)
        surface.render()
        glPopMatrix()

    


def switch_to_2d():
    nw,nh = _screen.dimensions
    w,h = _screen.width, _screen.height
    glViewport(0,0,w,h)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    glOrtho(-nw,nw,-nh,nh,1,0)
    #glOrtho(0,nw,0,nh,1,0)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
    setup_view_properties()

def switch_to_3d():
    w,h = _screen.width, _screen.height
    glViewport(0,0,w,h)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(45.0, float(w)/h,0.1,100.0)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
    glClearDepth(1.0)
    glEnable(GL_DEPTH_TEST)
    glDepthFunc(GL_LEQUAL)
    setup_view_properties()
    
    
def setup_view_properties():
    glShadeModel(GL_SMOOTH)
    glEnable(GL_TEXTURE_2D)
    glEnable(GL_BLEND)
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA)
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)

def set_mode((w,h)):
    assert w > 0 and h > 0
    if w < h:
        aspect = float(h)/float(w)
        n_w = 1.0
        n_h = n_w * aspect
    else:
        aspect = float(w)/float(h)
        n_h = 1.0
        n_w = n_h * aspect

    #print "Screen dimensions:", n_w, n_h
    
    global _screen
    _screen = Screen((w,h),(n_w,n_h))
    switch_to_2d()
    return _screen


def get_screen():
    return _screen


