import eventnet.driver
import eventnet._pygame
import nanothreads

import pygame
from pygame.locals import OPENGL, DOUBLEBUF, FULLSCREEN, NOFRAME, MOUSEMOTION

import graph
import display
import draw
import time
from vector import Vector

pygame.mixer.pre_init(44100,-16,2, 1024 * 3)
pygame.init()


def xy(position):
    position = Vector(position) * display._screen.ratio
    position = position - (display._screen.dimensions)
    position = position[0], -position[1]
    return Vector(position)
    
    
def xy_from_virtual(position):
    position = Vector((position[0], -position[1]))
    position = position + (display._screen.dimensions)
    position = position / display._screen.ratio
    return position.int()

class Widget(object):
    pass

class Button(Widget):
    def __init__(self):
        Widget.__init__(self)
        self.clicked = False
    def on_click(self, event):
        eventnet.driver.post('GUI_CLICK',button=self)
    def on_mousedown(self, event):
        self.scale -= 0.04
    def on_mouseup(self, event):
        self.scale += 0.04

class ShapeButton(Button):
    def __init__(self, shape):
        Button.__init__(self)
        self.shape = shape
    def __getattr__(self, key):
        return getattr(self.shape, key)


class Frame(eventnet.driver.Handler):
    def __init__(self):
        eventnet.driver.Handler.__init__(self)
        self.graph = graph.Graph()
        
    def register(self):
        pass

    def EVT_MouseButtonDown(self, event):
        hits = self.graph.pick(event.pos)
        try:
            item = hits[0]
        except IndexError:
            return

        if isinstance(item, Button):
            item.clicked = True
            item.on_mousedown(event)

    def EVT_MouseButtonUp(self, event):
        hits = self.graph.pick(event.pos)
        try:
            item = hits[0]
        except IndexError:
            pass
        else:
            if isinstance(item, Button):
                if item.clicked:
                    nanothreads.defer(item.on_click, event)

        for item in self.graph.items:
            if isinstance(item, Button):
                if item.clicked:
                    item.on_mouseup(event)
                    item.clicked = False


    def add(self, item, position=(0,0)):
        self.graph.add(item,position=position)
        item.graph = self.graph

    def remove(self, item):
        self.graph.remove(item)


class Application(eventnet.driver.Handler):
    def __init__(self, (w,h),icon=None,caption="LGT Application"):
        eventnet.driver.Handler.__init__(self)
        pygame.display.set_mode((w, h), OPENGL|DOUBLEBUF)
        display.set_mode((w, h))
        self.active_frame = None
        self.running = False
        nanothreads.register(eventnet._pygame.pump_events)
        nanothreads.register(self.poll)
        self.screen = display.get_screen()
        if icon:
            ico_surf = pygame.image.load(icon)
            pygame.display.set_icon(ico_surf)
        pygame.display.set_caption(caption)

    def EVT_Quit(self, event):
        self.running = False
        def exit():
            raise SystemExit
        nanothreads.defer(exit)

    def activate_frame(self, frame):
        if self.active_frame: 
            self.active_frame.release()
            self.active_frame.application = None
            
        self.active_frame = frame
        self.active_frame.application = self
        self.active_frame.capture()
        self.active_frame.register()

    def loop(self):
        nanothreads.loop()

    def poll(self):
        self.capture()
        self.running = True
        screen = display.get_screen()
        while self.running:
            time.sleep(0.01)
            if self.active_frame:
                screen.fill((0,0,0,0))
                screen.blit(self.active_frame.graph, (0,0))
                pygame.display.flip()
            yield None


class ScrollFrame(Frame):
    def __init__(self,speed=1):
        Frame.__init__(self)
        self.controls = graph.Graph()
        self.init_controls()
        self.__position = [0.0,0.0]
        self.x_move = 0
        self.y_move = 0
        self.speed = speed
        def get_position():
            last = time.clock()
            while True:
                now = time.clock()
                elapsed = now - last
                last = now
                inc = self.speed*elapsed
                self.__position[0] -= inc * self.x_move
                self.__position[1] -= inc * self.y_move
                if self.__position[0] < -1.33: self.__position[0] = -1.33
                if self.__position[0] > 1.33: self.__position[0] = 1.33
                if self.__position[1] < -1.0: self.__position[1] = -1.0
                if self.__position[1] > 1.0: self.__position[1] = 1.0
                if self.x_move != 0 or self.y_move !=0:
                    self.update(self.__position)
                yield None
        nanothreads.register(get_position)

    def init_controls(self):
        left = draw.Rectangle([(-0.1,1),(0.1,-1)])
        left.position = -1.3,0
        self.controls.add(left)
        self.left = left
        right = draw.Rectangle([(-0.1,1),(0.1,-1)])
        right.position = 1.3,0
        self.controls.add(right)
        self.right = right
        top = draw.Rectangle([(-1.33,0.1),(1.33,-0.1)])
        top.position = 0,1.0
        self.controls.add(top)
        self.top = top
        bottom = draw.Rectangle([(-1.33,0.1),(1.33,-0.1)])
        bottom.position = 0,-1.0
        self.controls.add(bottom)
        self.bottom = bottom

    def update(self, position):
        pass

    def EVT_MouseMotion(self, event):
        hits = self.controls.pick(event.pos)
        self.x_move = 0
        self.y_move = 0
        for hit in hits:
            if hit is self.left: self.x_move = -1
            if hit is self.top: self.y_move = 1
            if hit is self.right: self.x_move = 1
            if hit is self.bottom: self.y_move = -1

