import nanothreads
import random
import SiGL
import SiGL.actions
from SiGL.vector import Vector

class Mobile(object):
    def enable(self):
        nanothreads.register(self.loop)
        

class Hawk(Mobile):
    def __init__(self, textures, sounds):
        self.image = SiGL.draw.Image(textures['hawk.png'],position=[0,0])
        self.sound = sounds['hawk.wav']
        
    def loop(self):
        self.image.centre_point = -1,1
        self.image.scale = 0.5
        self.sound.set_volume(0.5)
        def change():
            th = SiGL.actions.Spin(self.image, degrees=360,duration=60,direction=1).execute()
            nanothreads.defer_for(random.uniform(1,60), self.sound.play)
            nanothreads.register_exit(th, change)
        change()
        yield None
    

class Cloud(Mobile):
    def __init__(self, textures, sounds):
        self.image = SiGL.draw.Image(textures['white_cloud.png'],position=[0,0],color = [1,1,1,0.2])
        self.shadow = SiGL.draw.Image(textures['black_cloud.png'],position=[0,0], scale=0.6,color=[1,1,1,0.3])
    def loop(self):
        
        def change():
            duration = random.uniform(20,50)
            self.image.rotation = random.uniform(0,360)
            self.image.position = Vector((random.uniform(-4,-2),random.uniform(-4,4)))
            self.shadow.position = self.image.position + (-0.5,-0.5)
            self.shadow.rotation = self.image.rotation
            destination = (4,random.uniform(-1,1))
            th = SiGL.actions.Move(self.shadow, position=destination, duration=duration).execute()
            th = SiGL.actions.Move(self.image, position=destination, duration=duration).execute()
            nanothreads.register_exit(th, change)
        change()
        yield None
        