from elementtree import ElementTree
import Image
import SiGL
import SiGL.actions
import SiGL.sound_actions
import SiGL.text
import nanothreads

import pygame.mixer

class Step(object):
    def __init__(self, start, repeat):
        self.start = start
        self.repeat = repeat
        self.actions = []
    
    def execute(self):
        print 'Starting action',self.start
        for action in self.actions:
            action.execute()
        if self.repeat:
            nanothreads.defer_for(self.repeat, self.execute)
        

class Scene(object):
    def __init__(self):
        self.textures = SiGL.texture.TexturePack()
        self.packs = []
        self.stringpack = SiGL.text.StringPack()
        self.images = {}
        self.graph = SiGL.graph.Graph()
        self.steps = []
        self.songs = []
        self.sounds = {}
        self.threads = []
        
    
    def play(self):
        play = False
        if self.songs:
            play = True
            pygame.mixer.music.load(self.songs.pop(0))
        for song in self.songs:
            pygame.mixer.music.queue(song)
        if play:
            pygame.mixer.music.play()
        for step in self.steps:
            self.threads.append(nanothreads.defer_for(step.start, step.execute))
            
    def stop(self):
        pygame.mixer.music.fadeout(5000)
        for thread in self.threads:
            nanothreads.kill(thread)
        for sound in self.sounds:
            self.sounds[sound].fadeout(1000)
        


def load(filename):
    scene = Scene()
    xml = ElementTree.parse(filename)
    
    for image in xml.findall('resources/image'):
        style = image.get("style")
        arguments = {}
        if style:
            arguments = eval("dict(%s)" % style)
        filename = image.get('filename')
        img = Image.open(filename)
        if img.size[0] > 800 or img.size[1] > 800:
            p = SiGL.texture.TexturePack()
            scene.packs.append(p)
            tex = p.add(img)
        else:
            tex = scene.textures.add(img)
        img = SiGL.draw.Image(tex, **arguments)
        scene.images[image.get("id")] = img
        img.position = 0,0
        scene.graph.add(img)
        
    for pack in scene.packs:
        pack.build()
    scene.textures.build()
    
    for image in xml.findall('resources/string'):
        style = image.get("style")
        arguments = {}
        if style:
            arguments = eval("dict(%s)" % style)
        tex = scene.stringpack.add(image.get("value"), **arguments)
        scene.images[image.get("id")] = tex
        tex.position = 0,0
        scene.graph.add(tex)
    
    for music in xml.findall('resources/music'):
        filename = music.get('filename')
        scene.songs.append(filename)
    
    for sound in xml.findall('resources/sound'):
        filename = sound.get('filename')
        scene.sounds[sound.get('id')] = pygame.mixer.Sound(filename)
        
    
    for element in xml.findall('timeline/step'):
        start_time = eval(element.get('time'))
        repeat = element.get('repeat')
        if repeat: repeat = eval(repeat)
        
        step = Step(start_time, repeat)
        
        
        for action in element.findall('action'):
            resource = action.get('resource')
            if resource is None: raise ValueError, 'No Resource specified.'
            method = action.get('type')
            if method is None: raise ValueError, 'No Action specified'
            style = action.get("style")
            arguments = {}
            if style:
                arguments = eval("dict(%s)" % style)
            
            if resource in scene.images:
                module = SiGL.actions
                source = scene.images
            elif resource in scene.sounds:
                module = SiGL.sound_actions
                source = scene.sounds
            else:
                print resource
                return 
            
            try:
                Action = getattr(module, method)
            except AttributeError:
                raise AttributeError, 'Invalid action type. (%s)' % method
            
            action = Action(source[resource], **arguments)
            step.actions.append(action)
        scene.steps.append(step)
                        
            
    return scene
